#!/opt/bin/perl -w

# (C) 1999 David C. Fox
# This code may be freely reproduced and modified, provided 
# 1. This copyright notice is retained, 
# and
# 2. substantial modifications are indicated, together with the author's
# name, in any redistributed versions.

use strict;
use English;

use File::Basename;
use File::Spec::Functions;

use ScanLatex ();
use LatexSpecial ();
use NatLatexParagraph ();
use OutFile ();

use ScanErrors;
use CurrentParagraph;
use newParseLatex;
use l2tRead;
use l2tProcess;


use WinWait;
use not_dead_yet;



##################################################
# main routine
##################################################

my $directive_marker = "\Q\%NLX\%\E";

my $unknown_macro_file;
my $unknown_math_file;

my $symbol_ref = {};
my $macro_ref = {};
my $data_ref = {};
my $phrases = new PhraseTree();
my $delimited = {};
my $parenthetical = {};
my $macro_tables = { 
  unknown => \$unknown_macro_file,
  symbols => $symbol_ref, 
  macros => $macro_ref, 
  data => $data_ref,
  phrases => $phrases,
  delimited => $delimited,
  parenthetical => $parenthetical
};

my $math_symbol_ref = {};
my $math_macro_ref = {};
my $math_data_ref = {};
my $math_phrases = new PhraseTree();
my $math_delimited = {};
my $math_parenthetical = {};
my $math_macro_tables = { 
  unknown => \$unknown_math_file,
  symbols => $math_symbol_ref, 
  macros => $math_macro_ref, 
  data => $math_data_ref,
  phrases => $math_phrases,
  delimited => $math_delimited,
  parenthetical => $math_parenthetical
};

my $input_filename;
my %command_line_options;


$input_filename = ReadCommandLine(\%command_line_options);
my $ignore = 0;
if ($command_line_options{"vocabulary builder"}) {
  $ignore = 1;
  if ( $command_line_options{"ignore"}) {
    print "Warning: -ignore should not be used with -vb\n";
    print "Resulting file may not be suitable for \n";
    print "NaturallySpeaking vocabulary builder\n";
  }
}

my ($without_suffix, $path, $suffix) = fileparse($input_filename, "\.tex");
$input_filename = "$path$without_suffix" . '.tex';

# find natural latex home directory

my $NaturalLatexHome = $ENV{"NLXHOME"};
if ( !defined($NaturalLatexHome)) {
  die_slowly("NLXHOME not set to NaturalLatex home directory");
}
my ($NLXVolume, $NLXPath) = File::Spec->splitpath($NaturalLatexHome, 1);

#  load translations/dispositions for macros

my $macro_list = File::Spec->catpath($NLXVolume, $NLXPath, "macro.nlx");

ReadMacroDispositions($macro_list, $directive_marker, $macro_tables);

my $specific_macro_list = $without_suffix . '.macro.nlx';
if (-e $specific_macro_list) {
  ReadMacroDispositions($specific_macro_list, $directive_marker, $macro_tables);
}

AddEndDispositions($macro_tables, $math_macro_tables);





my $math_list = File::Spec->catpath($NLXVolume, $NLXPath, "math.nlx");

ReadMacroDispositions($math_list, $directive_marker, $math_macro_tables);

my $specific_math_list = $without_suffix . '.math.nlx';
# print $specific_math_list;
if (-e $specific_math_list) {
# print $specific_math_list;
  ReadMacroDispositions($specific_math_list, $directive_marker,  
      $math_macro_tables);
}

AddEndMathDispositions($macro_tables, $math_macro_tables);

# print "$input_filename]\n";

my $input_file = new ScanLatex($input_filename);
if ( !(ref( $input_file))) {
  die_slowly($input_file);
}

my $temporary_filename;
my $output_filename;
my $old_filename;
if ( $command_line_options{"vocabulary builder"}) {
  $temporary_filename = $without_suffix . '.vb.working.txt';
  $output_filename = $without_suffix . '.vb.txt';
} else {
  $temporary_filename = $without_suffix . '.latex.working.txt';
  $output_filename = $without_suffix . '.latex.txt';
}

my $unknown_macro_filename = $without_suffix . '.unknown.macro';
my $unknown_math_filename = $without_suffix . '.unknown.math';

if (-e $temporary_filename) {
  unlink($temporary_filename) or 
      die_slowly("unable to delete old $temporary_filename\n");
}
my $output_file = new OutFile( $temporary_filename);
if ( !(ref( $output_file))) {
  die_slowly($output_file);
}
if ( $command_line_options{"ignore"}) {
  if (-e $unknown_macro_filename) {
    rename $unknown_macro_filename, "$unknown_macro_filename.old"
  }
  $unknown_macro_file = new OutFile( $unknown_macro_filename);
  if ( !(ref( $unknown_macro_file))) {
    die_slowly($unknown_macro_file);
  }
  if (-e $unknown_math_filename) {
    rename $unknown_math_filename, "$unknown_math_filename.old"
  }
  $unknown_math_file = new OutFile( $unknown_math_filename);
  if ( !(ref( $unknown_math_file))) {
    die_slowly($unknown_math_file);
  }
}

my $current_paragraph = new NatLatexParagraph($ignore);

ProcessLatex($input_file, $output_file, $current_paragraph,
    $directive_marker, \%command_line_options, 
    $macro_tables, $math_macro_tables, 0);

$output_file->close();
  
if (-e $output_filename) {
  my $old_filename = $output_filename . '.old';
  rename $output_filename, $old_filename
      or print "unable to re-name old file: $!\n";
}
if (!(rename $temporary_filename, $output_filename)) {
  print "unable to do re-naming working file $temporary_filename\n";
  print "to $output_filename: $!\n";
}


WinWait();


sub ReadCommandLine { # (\%command_line_options)
# filename = ReadCommandLine(\%command_line_options);
  my $options_ref = shift;

  my $input_filename = "";
  my $argument;
  my $usage = "Usage: l2t [-ignore] [-vb] filename.tex";
  my $error = 0;
  foreach $argument (@ARGV) {
    if ( $argument =~ s/^-//) {
      if ( $argument eq "ignore") {
        $options_ref->{"ignore"} = 1;
      } elsif ( $argument eq "vb") {
        $options_ref->{"vocabulary builder"} = 1;
      } else {
        print "unknown option $argument\n";
	$error = 1;
	last;
      }
    } else {
      $input_filename = $argument;
      last;
    }
  }
  if ( $input_filename ne "") {
    return $input_filename;
  }
  if ( !($error)) {
    print "missing filename\n";
  }
  die_slowly("$usage");
}

